@@ -40,7 +40,6 @@
     #define RCPP_STRING_DEBUG_3(fmt, M1, M2, M3)
 #endif
 
-
 namespace Rcpp {
 
     /**
@@ -53,7 +52,7 @@ namespace Rcpp {
         typedef internal::const_string_proxy<STRSXP> const_StringProxy;
 
         /** default constructor */
-        String(): data(Rf_mkChar("")), buffer(), valid(true), buffer_ready(true), enc(CE_NATIVE) {
+        String(): data(Rf_mkCharCE("", CE_UTF8)), buffer(), valid(true), buffer_ready(true), enc(CE_UTF8) {
             Rcpp_PreserveObject(data);
             RCPP_STRING_DEBUG("String()");
         }
@@ -64,25 +63,26 @@ namespace Rcpp {
             RCPP_STRING_DEBUG("String(const String&)");
         }
 
-        String(const String& other, const std::string& enc) : data(other.get_sexp()), valid(true), buffer_ready(false) {
-            Rcpp_PreserveObject(data);
-            set_encoding(enc);
-            RCPP_STRING_DEBUG("String(const String&)");
-        }
-
         /** construct a string from a single CHARSXP SEXP */
-        String(SEXP charsxp) : data(charsxp), valid(true), buffer_ready(false), enc(Rf_getCharCE(charsxp)) {
-            if (::Rf_isString(data) && ::Rf_length(data) != 1)
-                throw ::Rcpp::not_compatible("expecting a single value");
-            Rcpp_PreserveObject(data);
-            RCPP_STRING_DEBUG("String(SEXP)");
-        }
+        String(SEXP charsxp) : data(R_NilValue) {
+            if (TYPEOF(charsxp) == STRSXP) {
+                data = STRING_ELT(charsxp, 0);
+            } else if (TYPEOF(charsxp) == CHARSXP) {
+                data = charsxp;
+            }
 
-        String(SEXP charsxp, const std::string& enc) : data(charsxp), valid(true), buffer_ready(false) {
-            if (::Rf_isString(data) && ::Rf_length(data) != 1)
-                throw ::Rcpp::not_compatible("expecting a single value");
+            if (::Rf_isString(data) && ::Rf_length(data) != 1) {
+                const char* fmt = "Expecting a single string value: "
+                                  "[type=%s; extent=%i].";
+                throw ::Rcpp::not_compatible(fmt,
+                                             Rf_type2char(TYPEOF(data)),
+                                             ::Rf_length(data));
+            }
+
+            valid = true;
+            buffer_ready = false;
+            enc = Rf_getCharCE(data);
             Rcpp_PreserveObject(data);
-            set_encoding(enc);
             RCPP_STRING_DEBUG("String(SEXP)");
         }
 
@@ -92,10 +92,10 @@ namespace Rcpp {
             RCPP_STRING_DEBUG("String(const StringProxy&)");
         }
 
-        String(const StringProxy& proxy, const std::string& enc): data(proxy.get()), valid(true), buffer_ready(false) {
+        String(const StringProxy& proxy, cetype_t enc): data(proxy.get()), valid(true), buffer_ready(false) {
             Rcpp_PreserveObject(data);
             set_encoding(enc);
-            RCPP_STRING_DEBUG("String(const StringProxy&)");
+            RCPP_STRING_DEBUG("String(const StringProxy&, cetype_t)");
         }
 
         /** from string proxy */
@@ -104,40 +104,40 @@ namespace Rcpp {
             RCPP_STRING_DEBUG("String(const const_StringProxy&)");
         }
 
-        String(const const_StringProxy& proxy, const std::string& enc): data(proxy.get()), valid(true), buffer_ready(false) {
+        String(const const_StringProxy& proxy, cetype_t enc): data(proxy.get()), valid(true), buffer_ready(false) {
             Rcpp_PreserveObject(data);
             set_encoding(enc);
-            RCPP_STRING_DEBUG("String(const const_StringProxy&)");
+            RCPP_STRING_DEBUG("String(const const_StringProxy&, cetype_t)");
         }
 
         /** from a std::string */
-        String(const std::string& s) : buffer(s), valid(false), buffer_ready(true), enc(CE_NATIVE) {
+        String(const std::string& s, cetype_t enc = CE_UTF8) : buffer(s), valid(false), buffer_ready(true), enc(enc) {
             data = R_NilValue;
-            RCPP_STRING_DEBUG("String(const std::string&)");
+            RCPP_STRING_DEBUG("String(const std::string&, cetype_t)");
         }
 
-        String(const std::wstring& s) : data(internal::make_charsexp(s)), valid(true), buffer_ready(false), enc(CE_NATIVE) {
+        String(const std::wstring& s, cetype_t enc = CE_UTF8) : data(internal::make_charsexp(s)), valid(true), buffer_ready(false), enc(enc) {
             Rcpp_PreserveObject(data);
-            RCPP_STRING_DEBUG("String(const std::wstring&)");
+            RCPP_STRING_DEBUG("String(const std::wstring&, cetype_t)");
         }
 
         /** from a const char* */
-        String(const char* s) : buffer(s), valid(false), buffer_ready(true), enc(CE_NATIVE) {
+        String(const char* s, cetype_t enc = CE_UTF8) : buffer(s), valid(false), buffer_ready(true), enc(enc) {
             data = R_NilValue;
-            RCPP_STRING_DEBUG("String(const char*)");
+            RCPP_STRING_DEBUG("String(const char*, cetype_t)");
         }
 
-        String(const wchar_t* s) : data(internal::make_charsexp(s)), valid(true), buffer_ready(false), enc(CE_NATIVE) {
+        String(const wchar_t* s, cetype_t enc = CE_UTF8) : data(internal::make_charsexp(s)), valid(true), buffer_ready(false), enc(enc) {
             Rcpp_PreserveObject(data);
-            RCPP_STRING_DEBUG("String(const wchar_t* s)");
+            RCPP_STRING_DEBUG("String(const wchar_t* s, cetype_t)");
         }
 
         /** constructors from R primitives */
-        String(int x) : data(internal::r_coerce<INTSXP,STRSXP>(x)), valid(true), buffer_ready(false), enc(CE_NATIVE) {Rcpp_PreserveObject(data);}
-        String(double x) : data(internal::r_coerce<REALSXP,STRSXP>(x)), valid(true), buffer_ready(false), enc(CE_NATIVE) {Rcpp_PreserveObject(data);}
-        String(bool x) : data(internal::r_coerce<LGLSXP,STRSXP>(x)), valid(true) , buffer_ready(false), enc(CE_NATIVE) {Rcpp_PreserveObject(data);}
-        String(Rcomplex x) : data(internal::r_coerce<CPLXSXP,STRSXP>(x)), valid(true), buffer_ready(false), enc(CE_NATIVE) {Rcpp_PreserveObject(data);}
-        String(Rbyte x) : data(internal::r_coerce<RAWSXP,STRSXP>(x)), valid(true), buffer_ready(false), enc(CE_NATIVE) {Rcpp_PreserveObject(data);}
+        String(int x) : data(internal::r_coerce<INTSXP,STRSXP>(x)), valid(true), buffer_ready(false), enc(CE_UTF8) {Rcpp_PreserveObject(data);}
+        String(double x) : data(internal::r_coerce<REALSXP,STRSXP>(x)), valid(true), buffer_ready(false), enc(CE_UTF8) {Rcpp_PreserveObject(data);}
+        String(bool x) : data(internal::r_coerce<LGLSXP,STRSXP>(x)), valid(true) , buffer_ready(false), enc(CE_UTF8) {Rcpp_PreserveObject(data);}
+        String(Rcomplex x) : data(internal::r_coerce<CPLXSXP,STRSXP>(x)), valid(true), buffer_ready(false), enc(CE_UTF8) {Rcpp_PreserveObject(data);}
+        String(Rbyte x) : data(internal::r_coerce<RAWSXP,STRSXP>(x)), valid(true), buffer_ready(false), enc(CE_UTF8) {Rcpp_PreserveObject(data);}
 
         ~String() {
             Rcpp_ReleaseObject(data);
@@ -243,7 +243,8 @@ namespace Rcpp {
             RCPP_STRING_DEBUG_2("String::replace_first(const char* = '%s' , const char* = '%s')", s, news);
             if (is_na()) return *this;
             setBuffer();
-            size_t index = buffer.find_first_of(s);
+            std::string s2 = std::string(s);
+            size_t index = std::distance(buffer.begin(), std::search(buffer.begin(), buffer.end(), s2.begin(), s2.end()));
             if (index != std::string::npos) buffer.replace(index, strlen(s), news);
             valid = false;
             return *this;
@@ -268,7 +269,8 @@ namespace Rcpp {
             RCPP_STRING_DEBUG_2("String::replace_last(const char* = '%s' , const char* = '%s')", s, news);
             if (is_na()) return *this;
             setBuffer();
-            size_t index = buffer.find_last_of(s);
+            std::string s2 = std::string(s);
+            size_t index = std::distance(buffer.begin(), std::find_end(buffer.begin(), buffer.end(), s2.begin(), s2.end()));
             if (index != std::string::npos) buffer.replace(index, strlen(s), news);
             valid = false;
             return *this;
@@ -294,10 +296,13 @@ namespace Rcpp {
             RCPP_STRING_DEBUG_2("String::replace_all(const char* = '%s' , const char* = '%s')", s, news);
             if (is_na()) return *this;
             setBuffer();
-            size_t lens = strlen(s), len_news = strlen(news), index = buffer.find(s);
-            while(index != std::string::npos) {
-                buffer.replace(index, lens, news);
-                index = buffer.find(s, index + len_news);
+            std::string s2 = std::string(s);
+            std::string::iterator iter = buffer.begin();
+            while(true) {
+                iter = std::search(iter, buffer.end(), s2.begin(), s2.end());
+                if (iter == buffer.end()) break;
+                size_t index = std::distance(buffer.begin(), iter);
+                if (index != std::string::npos) buffer.replace(index, strlen(s), news);
             }
             valid = false;
             return *this;
@@ -382,17 +387,8 @@ namespace Rcpp {
             return buffer_ready ? buffer.c_str() : CHAR(data);
         }
 
-        inline const std::string get_encoding() const {
-            switch (enc) {
-                case CE_BYTES:
-                    return "bytes";
-                case CE_LATIN1:
-                    return "latin1";
-                case CE_UTF8:
-                    return "UTF-8";
-                default:
-                    return "unknown";
-            }
+        inline cetype_t get_encoding() const {
+            return enc;
         }
 
         inline void set_encoding(cetype_t encoding) {
@@ -407,18 +403,6 @@ namespace Rcpp {
             }
         }
 
-        inline void set_encoding(const std::string & encoding) {
-            if (encoding == "bytes") {
-                set_encoding(CE_BYTES);
-            } else if (encoding == "latin1") {
-                set_encoding(CE_LATIN1);
-            } else if (encoding == "UTF-8") {
-                set_encoding(CE_UTF8);
-            } else {
-                set_encoding(CE_ANY);
-            }
-        }
-
         bool operator<(const Rcpp::String& other) const {
             return strcmp(get_cstring(), other.get_cstring()) < 0;
         }
